@@ -23,18 +23,18 @@ import (
 	"github.com/cri-o/cri-o/internal/version"
 	libconfig "github.com/cri-o/cri-o/pkg/config"
 	"github.com/cri-o/cri-o/server"
-	v1 "github.com/cri-o/cri-o/server/cri/v1"
-	"github.com/cri-o/cri-o/server/cri/v1alpha2"
-	"github.com/cri-o/cri-o/server/metrics"
+	otel_collector "github.com/cri-o/cri-o/server/otel-collector"
 	"github.com/cri-o/cri-o/utils"
 	grpc_middleware "github.com/grpc-ecosystem/go-grpc-middleware"
 	"github.com/sirupsen/logrus"
 	"github.com/soheilhy/cmux"
+	"github.com/uptrace/opentelemetry-go-extra/otellogrus"
 	"github.com/urfave/cli/v2"
 	"go.opentelemetry.io/contrib/instrumentation/google.golang.org/grpc/otelgrpc"
 	sdktrace "go.opentelemetry.io/otel/sdk/trace"
 	"golang.org/x/sys/unix"
 	"google.golang.org/grpc"
+	v1 "k8s.io/cri-api/pkg/apis/runtime/v1"
 )
 
 func writeCrioGoroutineStacks() {
@@ -52,7 +52,7 @@ func catchShutdown(ctx context.Context, cancel context.CancelFunc, gserver *grpc
 	signal.Notify(sig, signals.Interrupt, signals.Term, unix.SIGUSR1, unix.SIGUSR2, unix.SIGPIPE, signals.Hup)
 	go func() {
 		for s := range sig {
-			logrus.WithFields(logrus.Fields{
+			log.WithFields(ctx, logrus.Fields{
 				"signal": s,
 			}).Debug("received signal")
 			switch s {
@@ -65,27 +65,27 @@ func catchShutdown(ctx context.Context, cancel context.CancelFunc, gserver *grpc
 			case unix.SIGPIPE:
 				continue
 			case signals.Interrupt:
-				logrus.Debugf("Caught SIGINT")
+				log.Debugf(ctx, "Caught SIGINT")
 			case signals.Term:
-				logrus.Debugf("Caught SIGTERM")
+				log.Debugf(ctx, "Caught SIGTERM")
 			default:
 				continue
 			}
 			*signalled = true
 			if tp != nil {
 				if err := tp.Shutdown(ctx); err != nil {
-					logrus.Warnf("Error shutting down opentelemetry tracer provider: %v", err)
+					log.Warnf(ctx, "Error shutting down opentelemetry tracer provider: %v", err)
 				}
 			}
 			gserver.GracefulStop()
 			hserver.Shutdown(ctx) // nolint: errcheck
 			if err := sserver.StopStreamServer(); err != nil {
-				logrus.Warnf("Error shutting down streaming server: %v", err)
+				log.Warnf(ctx, "Error shutting down streaming server: %v", err)
 			}
 			sserver.StopMonitors()
 			cancel()
 			if err := sserver.Shutdown(ctx); err != nil {
-				logrus.Warnf("Error shutting down main service %v", err)
+				log.Warnf(ctx, "Error shutting down main service %v", err)
 			}
 			return
 		}
@@ -172,6 +172,15 @@ func main() {
 		}
 		logrus.SetLevel(level)
 		logrus.AddHook(log.NewFilenameHook())
+		logrus.AddHook(otellogrus.NewHook(otellogrus.WithLevels(
+			logrus.PanicLevel,
+			logrus.FatalLevel,
+			logrus.ErrorLevel,
+			logrus.WarnLevel,
+			logrus.InfoLevel,
+			logrus.DebugLevel,
+			logrus.TraceLevel,
+		)))
 
 		filterHook, err := log.NewFilterHook(config.LogFilter)
 		if err != nil {
@@ -267,11 +276,11 @@ func main() {
 			logrus.Fatalf("Failed to chmod listen socket %s: %v", config.Listen, err)
 		}
 
-		var tracerProvider *sdktrace.TracerProvider
-		chainUnaryServer := grpc_middleware.ChainUnaryServer(metrics.UnaryInterceptor(), log.UnaryInterceptor())
-		chainStreamServer := grpc_middleware.ChainStreamServer(log.StreamInterceptor())
+		var (
+			tracerProvider *sdktrace.TracerProvider
+			opts           []otelgrpc.Option
+		)
 		if config.EnableTracing {
-			var opts []otelgrpc.Option
 			tracerProvider, opts, err = opentelemetry.InitTracing(
 				ctx,
 				config.TracingEndpoint,
@@ -280,16 +289,15 @@ func main() {
 			if err != nil {
 				logrus.Fatalf("Failed to initialize tracer provider: %v", err)
 			}
-			chainUnaryServer = grpc_middleware.ChainUnaryServer(
-				metrics.UnaryInterceptor(),
-				log.UnaryInterceptor(),
-				otelgrpc.UnaryServerInterceptor(opts...),
-			)
-			chainStreamServer = grpc_middleware.ChainStreamServer(log.StreamInterceptor(), otelgrpc.StreamServerInterceptor(opts...))
 		}
 		grpcServer := grpc.NewServer(
-			grpc.UnaryInterceptor(chainUnaryServer),
-			grpc.StreamInterceptor(chainStreamServer),
+			grpc.UnaryInterceptor(grpc_middleware.ChainUnaryServer(
+				otel_collector.UnaryInterceptor(),
+			)),
+			grpc.StreamInterceptor(grpc_middleware.ChainStreamServer(
+				otel_collector.StreamInterceptor(),
+			)),
+			grpc.StatsHandler(otelgrpc.NewServerHandler(opts...)),
 			grpc.MaxSendMsgSize(config.GRPCMaxSendMsgSize),
 			grpc.MaxRecvMsgSize(config.GRPCMaxRecvMsgSize),
 		)
@@ -317,9 +325,13 @@ func main() {
 				logrus.Error(err)
 			}
 
+			if err := os.MkdirAll(filepath.Dir(config.CleanShutdownSupportedFileName()), 0o755); err != nil {
+				logrus.Errorf("Creating clean shutdown supported parent directory: %v", err)
+			}
+
 			// Write "$CleanShutdownFile".supported to show crio-wipe that
 			// we should be wiping if the CleanShutdownFile wasn't found.
-			// This protects us from wiping after an upgrade from a version that don't support
+			// This protects us from wiping after an upgrade from a version that doesn't support
 			// CleanShutdownFile.
 			f, err := os.Create(config.CleanShutdownSupportedFileName())
 			if err != nil {
@@ -333,8 +345,15 @@ func main() {
 			}
 		}
 
-		v1alpha2.Register(grpcServer, crioServer)
-		v1.Register(grpcServer, crioServer)
+		// We always use 'Volatile: true' when creating containers, which means that in
+		// the event of an unclean shutdown, we might lose track of containers and layers.
+		// We need to call the garbage collection function to clean up the redundant files.
+		if err := crioServer.Store().GarbageCollect(); err != nil {
+			logrus.Errorf("Attempts to clean up unreferenced old container leftovers failed: %v", err)
+		}
+
+		v1.RegisterRuntimeServiceServer(grpcServer, crioServer)
+		v1.RegisterImageServiceServer(grpcServer, crioServer)
 
 		// after the daemon is done setting up we can notify systemd api
 		notifySystem()
