@@ -92,6 +92,9 @@ func mergeConfig(config *libconfig.Config, ctx *cli.Context) error {
 	if ctx.IsSet("signature-policy") {
 		config.SignaturePolicyPath = ctx.String("signature-policy")
 	}
+	if ctx.IsSet("signature-policy-dir") {
+		config.SignaturePolicyDir = ctx.String("signature-policy-dir")
+	}
 	if ctx.IsSet("root") {
 		config.Root = ctx.String("root")
 	}
@@ -332,6 +335,9 @@ func mergeConfig(config *libconfig.Config, ctx *cli.Context) error {
 	if ctx.IsSet("absent-mount-sources-to-reject") {
 		config.AbsentMountSourcesToReject = StringSliceTrySplit(ctx, "absent-mount-sources-to-reject")
 	}
+	if ctx.IsSet("irqbalance-config-restore-file") {
+		config.IrqBalanceConfigRestoreFile = ctx.String("irqbalance-config-restore-file")
+	}
 	if ctx.IsSet("internal-wipe") {
 		config.InternalWipe = ctx.Bool("internal-wipe")
 	}
@@ -398,6 +404,9 @@ func mergeConfig(config *libconfig.Config, ctx *cli.Context) error {
 	if ctx.IsSet("enable-pod-events") {
 		config.EnablePodEvents = ctx.Bool("enable-pod-events")
 	}
+	if ctx.IsSet("hostnetwork-disable-selinux") {
+		config.HostNetworkDisableSELinux = ctx.Bool("hostnetwork-disable-selinux")
+	}
 	return nil
 }
 
@@ -540,6 +549,13 @@ func getCrioFlags(defConf *libconfig.Config) []cli.Flag {
 			EnvVars:   []string{"CONTAINER_SIGNATURE_POLICY"},
 			TakesFile: true,
 		},
+		&cli.StringFlag{
+			Name:      "signature-policy-dir",
+			Usage:     "Path to the root directory for namespaced signature policies. Must be an absolute path.",
+			Value:     defConf.SignaturePolicyDir,
+			EnvVars:   []string{"CONTAINER_SIGNATURE_POLICY_DIR"},
+			TakesFile: true,
+		},
 		&cli.StringFlag{
 			Name:      "root",
 			Aliases:   []string{"r"},
@@ -618,7 +634,7 @@ func getCrioFlags(defConf *libconfig.Config) []cli.Flag {
 		},
 		&cli.BoolFlag{
 			Name:    "seccomp-use-default-when-empty",
-			Usage:   "Use the default seccomp profile when an empty one is specified.",
+			Usage:   "Use the default seccomp profile when an empty one is specified. This option is currently deprecated, and will be replaced by the SeccompDefault FeatureGate in Kubernetes.",
 			EnvVars: []string{"CONTAINER_SECCOMP_USE_DEFAULT_WHEN_EMPTY"},
 			Value:   defConf.Seccomp().UseDefaultWhenEmpty(),
 		},
@@ -1106,6 +1122,17 @@ func getCrioFlags(defConf *libconfig.Config) []cli.Flag {
 			Usage:   "If true, CRI-O starts sending the container events to the kubelet",
 			EnvVars: []string{"ENABLE_POD_EVENTS"},
 		},
+		&cli.StringFlag{
+			Name:  "irqbalance-config-restore-file",
+			Value: defConf.IrqBalanceConfigRestoreFile,
+			Usage: "Determines if CRI-O should attempt to restore the irqbalance config at startup with the mask in this file. Use the 'disable' value to disable the restore flow entirely.",
+		},
+		&cli.BoolFlag{
+			Name:    "hostnetwork-disable-selinux",
+			Usage:   "Determines whether SELinux should be disabled within a pod when it is running in the host network namespace.",
+			EnvVars: []string{"CONTAINER_HOSTNETWORK_DISABLE_SELINUX"},
+			Value:   defConf.HostNetworkDisableSELinux,
+		},
 	}
 }
 
@@ -1129,10 +1156,15 @@ func StringSliceTrySplit(ctx *cli.Context, name string) []string {
 			"Parsed commma separated CLI flag %q into dedicated values %v",
 			name, values,
 		)
-	} else {
-		// Copy the slice to avoid the cli flags being overwritten
-		values = append(values[:0:0], values...)
+
+		return values
 	}
 
-	return values
+	// Copy the slice to avoid the cli flags being overwritten
+	trimmedValues := []string{}
+	for _, value := range values {
+		trimmedValues = append(trimmedValues, strings.TrimSpace(value))
+	}
+
+	return trimmedValues
 }
