@@ -2,30 +2,31 @@ package lib
 
 import (
 	"context"
+	"errors"
 	"fmt"
 	"path/filepath"
 	"sync"
 	"time"
 
-	"github.com/containers/podman/v3/pkg/annotations"
-	"github.com/containers/podman/v3/pkg/hooks"
+	"github.com/containers/common/pkg/hooks"
+	"github.com/containers/podman/v4/pkg/annotations"
 	cstorage "github.com/containers/storage"
 	"github.com/containers/storage/pkg/ioutils"
 	"github.com/containers/storage/pkg/truncindex"
 	"github.com/cri-o/cri-o/internal/hostport"
 	"github.com/cri-o/cri-o/internal/lib/sandbox"
+	statsserver "github.com/cri-o/cri-o/internal/lib/stats"
 	"github.com/cri-o/cri-o/internal/log"
 	"github.com/cri-o/cri-o/internal/oci"
 	"github.com/cri-o/cri-o/internal/registrar"
 	"github.com/cri-o/cri-o/internal/storage"
 	crioann "github.com/cri-o/cri-o/pkg/annotations"
 	libconfig "github.com/cri-o/cri-o/pkg/config"
-	"github.com/cri-o/cri-o/server/cri/types"
 	json "github.com/json-iterator/go"
 	rspec "github.com/opencontainers/runtime-spec/specs-go"
 	"github.com/opencontainers/selinux/go-selinux/label"
-	"github.com/pkg/errors"
 	"github.com/sirupsen/logrus"
+	types "k8s.io/cri-api/pkg/apis/runtime/v1"
 )
 
 // ContainerManagerCRIO specifies an annotation value which indicates that the
@@ -44,6 +45,7 @@ type ContainerServer struct {
 	podNameIndex         *registrar.Registrar
 	podIDIndex           *truncindex.TruncIndex
 	Hooks                *hooks.Manager
+	*statsserver.StatsServer
 
 	stateLock sync.Locker
 	state     *containerServerState
@@ -117,7 +119,7 @@ func New(ctx context.Context, configIface libconfig.Iface) (*ContainerServer, er
 		return nil, err
 	}
 
-	return &ContainerServer{
+	c := &ContainerServer{
 		runtime:              runtime,
 		store:                store,
 		storageImageServer:   imageService,
@@ -135,22 +137,26 @@ func New(ctx context.Context, configIface libconfig.Iface) (*ContainerServer, er
 			processLevels:   make(map[string]int),
 		},
 		config: config,
-	}, nil
+	}
+	c.StatsServer = statsserver.New(c)
+	return c, nil
 }
 
 // LoadSandbox loads a sandbox from the disk into the sandbox store
 func (c *ContainerServer) LoadSandbox(ctx context.Context, id string) (sb *sandbox.Sandbox, retErr error) {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	config, err := c.store.FromContainerDirectory(id, "config.json")
 	if err != nil {
 		return nil, err
 	}
 	var m rspec.Spec
 	if err := json.Unmarshal(config, &m); err != nil {
-		return nil, errors.Wrap(err, "error unmarshalling sandbox spec")
+		return nil, fmt.Errorf("error unmarshalling sandbox spec: %w", err)
 	}
 	labels := make(map[string]string)
 	if err := json.Unmarshal([]byte(m.Annotations[annotations.Labels]), &labels); err != nil {
-		return nil, errors.Wrapf(err, "error unmarshalling %s annotation", annotations.Labels)
+		return nil, fmt.Errorf("error unmarshalling %s annotation: %w", annotations.Labels, err)
 	}
 	name := m.Annotations[annotations.Name]
 	name, err = c.ReservePodName(id, name)
@@ -162,9 +168,9 @@ func (c *ContainerServer) LoadSandbox(ctx context.Context, id string) (sb *sandb
 			c.ReleasePodName(name)
 		}
 	}()
-	var metadata sandbox.Metadata
+	var metadata types.PodSandboxMetadata
 	if err := json.Unmarshal([]byte(m.Annotations[annotations.Metadata]), &metadata); err != nil {
-		return nil, errors.Wrapf(err, "error unmarshalling %s annotation", annotations.Metadata)
+		return nil, fmt.Errorf("error unmarshalling %s annotation: %w", annotations.Metadata, err)
 	}
 
 	processLabel := m.Process.SelinuxLabel
@@ -174,24 +180,24 @@ func (c *ContainerServer) LoadSandbox(ctx context.Context, id string) (sb *sandb
 
 	kubeAnnotations := make(map[string]string)
 	if err := json.Unmarshal([]byte(m.Annotations[annotations.Annotations]), &kubeAnnotations); err != nil {
-		return nil, errors.Wrapf(err, "error unmarshalling %s annotation", annotations.Annotations)
+		return nil, fmt.Errorf("error unmarshalling %s annotation: %w", annotations.Annotations, err)
 	}
 
 	portMappings := []*hostport.PortMapping{}
 	if err := json.Unmarshal([]byte(m.Annotations[annotations.PortMappings]), &portMappings); err != nil {
-		return nil, errors.Wrapf(err, "error unmarshalling %s annotation", annotations.PortMappings)
+		return nil, fmt.Errorf("error unmarshalling %s annotation: %w", annotations.PortMappings, err)
 	}
 
 	privileged := isTrue(m.Annotations[annotations.PrivilegedRuntime])
 	hostNetwork := isTrue(m.Annotations[annotations.HostNetwork])
 	nsOpts := types.NamespaceOption{}
 	if err := json.Unmarshal([]byte(m.Annotations[annotations.NamespaceOptions]), &nsOpts); err != nil {
-		return nil, errors.Wrapf(err, "error unmarshalling %s annotation", annotations.NamespaceOptions)
+		return nil, fmt.Errorf("error unmarshalling %s annotation: %w", annotations.NamespaceOptions, err)
 	}
 
 	created, err := time.Parse(time.RFC3339Nano, m.Annotations[annotations.Created])
 	if err != nil {
-		return nil, errors.Wrap(err, "parsing created timestamp annotation")
+		return nil, fmt.Errorf("parsing created timestamp annotation: %w", err)
 	}
 
 	sb, err = sandbox.New(id, m.Annotations[annotations.Namespace], name, m.Annotations[annotations.KubeName], filepath.Dir(m.Annotations[annotations.LogPath]), labels, kubeAnnotations, processLabel, mountLabel, &metadata, m.Annotations[annotations.ShmPath], m.Annotations[annotations.CgroupParent], privileged, m.Annotations[annotations.RuntimeHandler], m.Annotations[annotations.ResolvPath], m.Annotations[annotations.HostName], portMappings, hostNetwork, created, m.Annotations[crioann.UsernsModeAnnotation])
@@ -209,13 +215,13 @@ func (c *ContainerServer) LoadSandbox(ctx context.Context, id string) (sb *sandb
 			}
 		}
 	}()
-	if err := c.AddSandbox(sb); err != nil {
+	if err := c.AddSandbox(ctx, sb); err != nil {
 		return sb, err
 	}
 
 	defer func() {
 		if retErr != nil {
-			if err := c.RemoveSandbox(sb.ID()); err != nil {
+			if err := c.RemoveSandbox(ctx, sb.ID()); err != nil {
 				log.Warnf(ctx, "Could not remove sandbox ID %s: %v", sb.ID(), err)
 			}
 		}
@@ -239,7 +245,7 @@ func (c *ContainerServer) LoadSandbox(ctx context.Context, id string) (sb *sandb
 	}
 	defer func() {
 		if retErr != nil {
-			c.ReleaseContainerName(cname)
+			c.ReleaseContainerName(ctx, cname)
 		}
 	}()
 
@@ -266,7 +272,7 @@ func (c *ContainerServer) LoadSandbox(ctx context.Context, id string) (sb *sandb
 	if m.Annotations[annotations.Volumes] != "" {
 		containerVolumes := []oci.ContainerVolume{}
 		if err = json.Unmarshal([]byte(m.Annotations[annotations.Volumes]), &containerVolumes); err != nil {
-			return sb, fmt.Errorf("failed to unmarshal container volumes: %v", err)
+			return sb, fmt.Errorf("failed to unmarshal container volumes: %w", err)
 		}
 		for _, cv := range containerVolumes {
 			scontainer.AddVolume(cv)
@@ -277,6 +283,9 @@ func (c *ContainerServer) LoadSandbox(ctx context.Context, id string) (sb *sandb
 		return sb, err
 	}
 
+	// We should restore the infraContainer to the container state store
+	c.AddInfraContainer(ctx, scontainer)
+
 	sb.RestoreStopped()
 	// We add an NS only if we can load a permanent one.
 	// Otherwise, the sandbox will live in the host namespace.
@@ -299,13 +308,13 @@ func (c *ContainerServer) LoadSandbox(ctx context.Context, id string) (sb *sandb
 	}
 
 	if err := c.ContainerStateFromDisk(ctx, scontainer); err != nil {
-		return sb, fmt.Errorf("error reading sandbox state from disk %q: %v", scontainer.ID(), err)
+		return sb, fmt.Errorf("error reading sandbox state from disk %q: %w", scontainer.ID(), err)
 	}
 
 	// We write back the state because it is possible that crio did not have a chance to
 	// read the exit file and persist exit code into the state on reboot.
 	if err := c.ContainerStateToDisk(ctx, scontainer); err != nil {
-		return sb, fmt.Errorf("failed to write container %q state to disk: %v", scontainer.ID(), err)
+		return sb, fmt.Errorf("failed to write container %q state to disk: %w", scontainer.ID(), err)
 	}
 
 	sb.SetCreated()
@@ -349,6 +358,8 @@ var ErrIsNonCrioContainer = errors.New("non CRI-O container")
 
 // LoadContainer loads a container from the disk into the container store
 func (c *ContainerServer) LoadContainer(ctx context.Context, id string) (retErr error) {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	config, err := c.store.FromContainerDirectory(id, "config.json")
 	if err != nil {
 		return err
@@ -375,11 +386,11 @@ func (c *ContainerServer) LoadContainer(ctx context.Context, id string) (retErr
 
 	defer func() {
 		if retErr != nil {
-			c.ReleaseContainerName(name)
+			c.ReleaseContainerName(ctx, name)
 		}
 	}()
 
-	var metadata oci.Metadata
+	var metadata types.ContainerMetadata
 	if err := json.Unmarshal([]byte(m.Annotations[annotations.Metadata]), &metadata); err != nil {
 		return err
 	}
@@ -417,6 +428,11 @@ func (c *ContainerServer) LoadContainer(ctx context.Context, id string) (retErr
 		imgRef = ""
 	}
 
+	platformRuntimePath, ok := m.Annotations[crioann.PlatformRuntimePath]
+	if !ok {
+		platformRuntimePath = ""
+	}
+
 	kubeAnnotations := make(map[string]string)
 	if err := json.Unmarshal([]byte(m.Annotations[annotations.Annotations]), &kubeAnnotations); err != nil {
 		return err
@@ -437,17 +453,19 @@ func (c *ContainerServer) LoadContainer(ctx context.Context, id string) (retErr
 	ctr.SetSeccompProfilePath(spp)
 
 	if err := c.ContainerStateFromDisk(ctx, ctr); err != nil {
-		return fmt.Errorf("error reading container state from disk %q: %v", ctr.ID(), err)
+		return fmt.Errorf("error reading container state from disk %q: %w", ctr.ID(), err)
 	}
 
 	// We write back the state because it is possible that crio did not have a chance to
 	// read the exit file and persist exit code into the state on reboot.
 	if err := c.ContainerStateToDisk(ctx, ctr); err != nil {
-		return fmt.Errorf("failed to write container state to disk %q: %v", ctr.ID(), err)
+		return fmt.Errorf("failed to write container state to disk %q: %w", ctr.ID(), err)
 	}
 	ctr.SetCreated()
 
-	c.AddContainer(ctr)
+	ctr.SetRuntimePathForPlatform(platformRuntimePath)
+
+	c.AddContainer(ctx, ctr)
 
 	return c.ctrIDIndex.Add(id)
 }
@@ -459,6 +477,8 @@ func isTrue(annotaton string) bool {
 // ContainerStateFromDisk retrieves information on the state of a running container
 // from the disk
 func (c *ContainerServer) ContainerStateFromDisk(ctx context.Context, ctr *oci.Container) error {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	if err := ctr.FromDisk(); err != nil {
 		return err
 	}
@@ -472,6 +492,8 @@ func (c *ContainerServer) ContainerStateFromDisk(ctx context.Context, ctr *oci.C
 // ContainerStateToDisk writes the container's state information to a JSON file
 // on disk
 func (c *ContainerServer) ContainerStateToDisk(ctx context.Context, ctr *oci.Container) error {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	if err := c.Runtime().UpdateContainerStatus(ctx, ctr); err != nil {
 		log.Warnf(ctx, "Error updating the container status %q: %v", ctr.ID(), err)
 	}
@@ -488,7 +510,7 @@ func (c *ContainerServer) ContainerStateToDisk(ctx context.Context, ctr *oci.Con
 // ReserveContainerName holds a name for a container that is being created
 func (c *ContainerServer) ReserveContainerName(id, name string) (string, error) {
 	if err := c.ctrNameIndex.Reserve(name, id); err != nil {
-		err = fmt.Errorf("error reserving ctr name %s for id %s: %v", name, id, err)
+		err = fmt.Errorf("error reserving ctr name %s for id %s: %w", name, id, err)
 		logrus.Warn(err)
 		return "", err
 	}
@@ -502,14 +524,16 @@ func (c *ContainerServer) ContainerIDForName(name string) (string, error) {
 
 // ReleaseContainerName releases a container name from the index so that it can
 // be used by other containers
-func (c *ContainerServer) ReleaseContainerName(name string) {
+func (c *ContainerServer) ReleaseContainerName(ctx context.Context, name string) {
+	_, span := log.StartSpan(ctx)
+	defer span.End()
 	c.ctrNameIndex.Release(name)
 }
 
 // ReservePodName holds a name for a pod that is being created
 func (c *ContainerServer) ReservePodName(id, name string) (string, error) {
 	if err := c.podNameIndex.Reserve(name, id); err != nil {
-		err = fmt.Errorf("error reserving pod name %s for id %s: %v", name, id, err)
+		err = fmt.Errorf("error reserving pod name %s for id %s: %w", name, id, err)
 		logrus.Warn(err)
 		return "", err
 	}
@@ -542,6 +566,7 @@ func (c *ContainerServer) Shutdown() error {
 	if err != nil && !errors.Is(err, cstorage.ErrLayerUsedByContainer) {
 		return err
 	}
+	c.StatsServer.Shutdown()
 	return nil
 }
 
@@ -554,27 +579,31 @@ type containerServerState struct {
 }
 
 // AddContainer adds a container to the container state store
-func (c *ContainerServer) AddContainer(ctr *oci.Container) {
+func (c *ContainerServer) AddContainer(ctx context.Context, ctr *oci.Container) {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	newSandbox := c.state.sandboxes.Get(ctr.Sandbox())
 	if newSandbox == nil {
 		return
 	}
-	newSandbox.AddContainer(ctr)
+	newSandbox.AddContainer(ctx, ctr)
 	c.state.containers.Add(ctr.ID(), ctr)
 }
 
 // AddInfraContainer adds a container to the container state store
-func (c *ContainerServer) AddInfraContainer(ctr *oci.Container) {
+func (c *ContainerServer) AddInfraContainer(ctx context.Context, ctr *oci.Container) {
 	c.state.infraContainers.Add(ctr.ID(), ctr)
 }
 
 // GetContainer returns a container by its ID
-func (c *ContainerServer) GetContainer(id string) *oci.Container {
+func (c *ContainerServer) GetContainer(ctx context.Context, id string) *oci.Container {
 	return c.state.containers.Get(id)
 }
 
 // GetInfraContainer returns a container by its ID
-func (c *ContainerServer) GetInfraContainer(id string) *oci.Container {
+func (c *ContainerServer) GetInfraContainer(ctx context.Context, id string) *oci.Container {
+	_, span := log.StartSpan(ctx)
+	defer span.End()
 	return c.state.infraContainers.Get(id)
 }
 
@@ -584,18 +613,26 @@ func (c *ContainerServer) HasContainer(id string) bool {
 }
 
 // RemoveContainer removes a container from the container state store
-func (c *ContainerServer) RemoveContainer(ctr *oci.Container) {
+func (c *ContainerServer) RemoveContainer(ctx context.Context, ctr *oci.Container) {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	sbID := ctr.Sandbox()
 	sb := c.state.sandboxes.Get(sbID)
 	if sb == nil {
 		return
 	}
-	sb.RemoveContainer(ctr)
+	sb.RemoveContainer(ctx, ctr)
+	c.RemoveStatsForContainer(ctr)
+	if err := ctr.RemoveManagedPIDNamespace(); err != nil {
+		log.Errorf(ctx, "Failed to remove container %s PID namespace: %v", ctr.ID(), err)
+	}
 	c.state.containers.Delete(ctr.ID())
 }
 
 // RemoveInfraContainer removes a container from the container state store
-func (c *ContainerServer) RemoveInfraContainer(ctr *oci.Container) {
+func (c *ContainerServer) RemoveInfraContainer(ctx context.Context, ctr *oci.Container) {
+	_, span := log.StartSpan(ctx)
+	defer span.End()
 	c.state.infraContainers.Delete(ctr.ID())
 }
 
@@ -624,7 +661,9 @@ func (c *ContainerServer) ListContainers(filters ...func(*oci.Container) bool) (
 }
 
 // AddSandbox adds a sandbox to the sandbox state store
-func (c *ContainerServer) AddSandbox(sb *sandbox.Sandbox) error {
+func (c *ContainerServer) AddSandbox(ctx context.Context, sb *sandbox.Sandbox) error {
+	_, span := log.StartSpan(ctx)
+	defer span.End()
 	c.state.sandboxes.Add(sb.ID(), sb)
 
 	c.stateLock.Lock()
@@ -652,7 +691,9 @@ func (c *ContainerServer) HasSandbox(id string) bool {
 }
 
 // RemoveSandbox removes a sandbox from the state store
-func (c *ContainerServer) RemoveSandbox(id string) error {
+func (c *ContainerServer) RemoveSandbox(ctx context.Context, id string) error {
+	_, span := log.StartSpan(ctx)
+	defer span.End()
 	sb := c.state.sandboxes.Get(id)
 	if sb == nil {
 		return nil
@@ -664,6 +705,7 @@ func (c *ContainerServer) RemoveSandbox(id string) error {
 		return err
 	}
 
+	c.RemoveStatsForSandbox(sb)
 	c.state.sandboxes.Delete(id)
 	return nil
 }
@@ -673,17 +715,6 @@ func (c *ContainerServer) ListSandboxes() []*sandbox.Sandbox {
 	return c.state.sandboxes.List()
 }
 
-// StopContainerAndWait is a wrapping function that stops a container and waits for the container state to be stopped
-func (c *ContainerServer) StopContainerAndWait(ctx context.Context, ctr *oci.Container, timeout int64) error {
-	if err := c.Runtime().StopContainer(ctx, ctr, timeout); err != nil {
-		return fmt.Errorf("failed to stop container %s: %v", ctr.Name(), err)
-	}
-	if err := c.Runtime().WaitContainerStateStopped(ctx, ctr); err != nil {
-		return fmt.Errorf("failed to get container 'stopped' status %s: %v", ctr.Name(), err)
-	}
-	return nil
-}
-
 func (c *ContainerServer) UpdateContainerLinuxResources(ctr *oci.Container, resources *rspec.LinuxResources) {
 	updatedSpec := ctr.Spec()
 	if updatedSpec.Linux == nil {
