@@ -71,14 +71,15 @@ type Container struct {
 	created            bool
 	spoofed            bool
 	stopping           bool
-	stopTimeoutChan    chan time.Duration
-	stoppedChan        chan struct{}
-	stopStoppingChan   chan struct{}
 	stopLock           sync.Mutex
+	stopTimeoutChan    chan int64
+	stopWatchers       []chan struct{}
 	pidns              nsmgr.Namespace
 	restore            bool
 	restoreArchive     string
 	restoreIsOCIImage  bool
+	resources          *types.ContainerResources
+	runtimePath        string // runtime path for a given platform
 }
 
 func (c *Container) CRIAttributes() *types.ContainerAttributes {
@@ -114,6 +115,8 @@ type ContainerState struct {
 	// This is used to track whether the PID we have stored
 	// is the same as the corresponding PID on the host.
 	InitStartTime string `json:"initStartTime,omitempty"`
+	// Checkpoint/Restore related states
+	CheckpointedAt time.Time `json:"checkpointedTime,omitempty"`
 }
 
 // NewContainer creates a container object.
@@ -133,21 +136,20 @@ func NewContainer(id, name, bundlePath, logPath string, labels, crioAnnotations,
 			},
 			ImageRef: imageRef,
 		},
-		name:             name,
-		bundlePath:       bundlePath,
-		logPath:          logPath,
-		terminal:         terminal,
-		stdin:            stdin,
-		stdinOnce:        stdinOnce,
-		runtimeHandler:   runtimeHandler,
-		crioAnnotations:  crioAnnotations,
-		imageName:        imageName,
-		dir:              dir,
-		state:            state,
-		stopSignal:       stopSignal,
-		stopTimeoutChan:  make(chan time.Duration, 1),
-		stoppedChan:      make(chan struct{}, 1),
-		stopStoppingChan: make(chan struct{}, 1),
+		name:            name,
+		bundlePath:      bundlePath,
+		logPath:         logPath,
+		terminal:        terminal,
+		stdin:           stdin,
+		stdinOnce:       stdinOnce,
+		runtimeHandler:  runtimeHandler,
+		crioAnnotations: crioAnnotations,
+		imageName:       imageName,
+		dir:             dir,
+		state:           state,
+		stopSignal:      stopSignal,
+		stopTimeoutChan: make(chan int64, 10),
+		stopWatchers:    []chan struct{}{},
 	}
 	return c, nil
 }
@@ -200,6 +202,7 @@ func (c *Container) CRIContainer() *types.Container {
 // SetSpec loads the OCI spec in the container struct
 func (c *Container) SetSpec(s *specs.Spec) {
 	c.spec = s
+	c.SetResources(s)
 }
 
 // Spec returns a copy of the spec for the container
@@ -296,6 +299,16 @@ func (c *Container) CreatedAt() time.Time {
 	return c.state.Created
 }
 
+// CheckpointedAt returns the container checkpoint time
+func (c *Container) CheckpointedAt() time.Time {
+	return c.state.CheckpointedAt
+}
+
+// SetCheckpointedAt sets the time of checkpointing
+func (c *Container) SetCheckpointedAt(checkpointedAt time.Time) {
+	c.state.CheckpointedAt = checkpointedAt
+}
+
 // Name returns the name of the container.
 func (c *Container) Name() string {
 	return c.name
@@ -485,9 +498,9 @@ func (c *Container) exitFilePath() string {
 	return filepath.Join(c.dir, "exit")
 }
 
-// IsAlive is a function that checks if a container's init PID exists.
+// Living is a function that checks if a container's init PID exists.
 // It is used to check a container state when we don't want a `$runtime state` call
-func (c *Container) IsAlive() error {
+func (c *Container) Living() error {
 	if _, err := c.pid(); err != nil {
 		return fmt.Errorf("checking if PID of %s is running failed: %w", c.ID(), err)
 	}
@@ -589,7 +602,7 @@ func GetPidStartTimeFromFile(file string) (string, error) {
 // a container is not stoppable if it's paused or stopped
 // if it's paused, that's an error, and is reported as such
 func (c *Container) ShouldBeStopped() error {
-	switch c.state.Status {
+	switch c.State().Status {
 	case ContainerStateStopped: // no-op
 		return ErrContainerStopped
 	case ContainerStatePaused:
@@ -607,40 +620,44 @@ func (c *Container) Spoofed() bool {
 }
 
 // SetAsStopping marks a container as being stopped.
-// If a stop is currently happening, it also sends the new timeout
-// along the stopTimeoutChan, allowing the in-progress stop
-// to stop faster, or ignore the new stop timeout.
-// In this case, it also returns true, signifying the caller doesn't have to
-// Do any stop related cleanup, as the original caller (alreadyStopping=false)
-// will do said cleanup.
-func (c *Container) SetAsStopping(timeout int64) (alreadyStopping bool) {
-	// First, need to check if the container is already stopping
+// Returns true if the container was not set as stopping before, and false otherwise (i.e. on subsequent calls)."
+func (c *Container) SetAsStopping() (setToStopping bool) {
 	c.stopLock.Lock()
 	defer c.stopLock.Unlock()
-	if c.stopping {
-		// If so, we shouldn't wait forever on the opLock.
-		// This can cause issues where the container stop gets DOSed by a very long
-		// timeout, followed a shorter one coming in.
-		// Instead, interrupt the other stop with this new one.
-		select {
-		case c.stopTimeoutChan <- time.Duration(timeout) * time.Second:
-		case <-c.stoppedChan: // This case is to avoid waiting forever once another routine has finished.
-		case <-c.stopStoppingChan: // This case is to avoid deadlocking with SetAsNotStopping.
-		}
+	if !c.stopping {
+		c.stopping = true
 		return true
 	}
-	// Regardless, set the container as actively stopping.
-	c.stopping = true
-	// And reset the stopStoppingChan
-	c.stopStoppingChan = make(chan struct{}, 1)
 	return false
 }
 
-// SetAsNotStopping unsets the stopping field indicating to new callers that the container
-// is no longer actively stopping.
-func (c *Container) SetAsNotStopping() {
+func (c *Container) WaitOnStopTimeout(ctx context.Context, timeout int64) {
 	c.stopLock.Lock()
+	if !c.stopping {
+		c.stopLock.Unlock()
+		return
+	}
+
+	c.stopTimeoutChan <- timeout
+
+	watcher := make(chan struct{}, 1)
+	c.stopWatchers = append(c.stopWatchers, watcher)
+	c.stopLock.Unlock()
+
+	select {
+	case <-ctx.Done():
+	case <-watcher:
+	}
+}
+
+func (c *Container) SetAsDoneStopping() {
+	c.stopLock.Lock()
+	for _, watcher := range c.stopWatchers {
+		close(watcher)
+	}
+	c.stopWatchers = make([]chan struct{}, 0)
 	c.stopping = false
+	close(c.stopTimeoutChan)
 	c.stopLock.Unlock()
 }
 
@@ -698,3 +715,68 @@ func (c *Container) RestoreIsOCIImage() bool {
 func (c *Container) SetRestoreIsOCIImage(restoreIsOCIImage bool) {
 	c.restoreIsOCIImage = restoreIsOCIImage
 }
+
+// SetResources loads the OCI Spec.Linux.Resources in the container struct
+func (c *Container) SetResources(s *specs.Spec) {
+	if s.Linux != nil && s.Linux.Resources != nil {
+		linuxResources := s.Linux.Resources
+		resourceStatus := &types.ContainerResources{}
+		resourceStatus.Linux = &types.LinuxContainerResources{}
+		if linuxResources.CPU != nil {
+			resourceStatus.Linux.CpusetCpus = linuxResources.CPU.Cpus
+			resourceStatus.Linux.CpusetMems = linuxResources.CPU.Mems
+			if linuxResources.CPU.Period != nil {
+				resourceStatus.Linux.CpuPeriod = int64(*linuxResources.CPU.Period)
+			}
+			if linuxResources.CPU.Quota != nil {
+				resourceStatus.Linux.CpuQuota = *linuxResources.CPU.Quota
+			}
+			if linuxResources.CPU.Shares != nil {
+				resourceStatus.Linux.CpuShares = int64(*linuxResources.CPU.Shares)
+			}
+		}
+		if linuxResources.Memory != nil {
+			if linuxResources.Memory.Limit != nil {
+				resourceStatus.Linux.MemoryLimitInBytes = *linuxResources.Memory.Limit
+			}
+			if linuxResources.Memory.Swap != nil {
+				resourceStatus.Linux.MemorySwapLimitInBytes = *linuxResources.Memory.Swap
+			}
+		}
+		if s.Process != nil {
+			if s.Process.OOMScoreAdj != nil {
+				resourceStatus.Linux.OomScoreAdj = int64(*s.Process.OOMScoreAdj)
+			}
+		}
+		resourceStatus.Linux.Unified = make(map[string]string)
+		for key, value := range linuxResources.Unified {
+			resourceStatus.Linux.Unified[key] = value
+		}
+		resourceStatus.Linux.HugepageLimits = []*types.HugepageLimit{}
+		for _, entry := range linuxResources.HugepageLimits {
+			resourceStatus.Linux.HugepageLimits = append(resourceStatus.Linux.HugepageLimits, &types.HugepageLimit{
+				PageSize: entry.Pagesize,
+				Limit:    entry.Limit,
+			})
+		}
+		c.resources = resourceStatus
+	}
+}
+
+// GetResources returns a copy of the Linux resources from Container
+func (c *Container) GetResources() *types.ContainerResources {
+	return c.resources
+}
+
+// SetRuntimePathForPlatform sets the runtime path for a given platform.
+func (c *Container) SetRuntimePathForPlatform(runtimePath string) {
+	c.runtimePath = runtimePath
+}
+
+// RuntimePathForPlatform returns the runtime path for a given platform.
+func (c *Container) RuntimePathForPlatform(r *runtimeOCI) string {
+	if c.runtimePath == "" {
+		return r.handler.RuntimePath
+	}
+	return c.runtimePath
+}
