@@ -103,7 +103,7 @@ const (
 	// ImageVolumesBind option is for using bind mounted volumes
 	ImageVolumesBind ImageVolumesType = "bind"
 	// DefaultPauseImage is default pause image
-	DefaultPauseImage string = "registry.k8s.io/pause:3.6"
+	DefaultPauseImage string = "registry.k8s.io/pause:3.9"
 )
 
 const (
@@ -124,6 +124,8 @@ const (
 const (
 	// DefaultIrqBalanceConfigFile default irqbalance service configuration file path
 	DefaultIrqBalanceConfigFile = "/etc/sysconfig/irqbalance"
+	// DefaultIrqBalanceConfigRestoreFile contains the banned cpu mask configuration to restore. Name due to backward compatibility.
+	DefaultIrqBalanceConfigRestoreFile = "/etc/sysconfig/orig_irq_banned_cpus"
 )
 
 // This structure is necessary to fake the TOML tables when parsing,
@@ -212,6 +214,10 @@ type RuntimeHandler struct {
 
 	// MonitorExecCgroup indicates whether to move exec probes to the container's cgroup.
 	MonitorExecCgroup string `toml:"monitor_exec_cgroup,omitempty"`
+
+	// PlatformRuntimePaths defines a configuration option that specifies
+	// the runtime paths for different platforms.
+	PlatformRuntimePaths map[string]string `toml:"platform_runtime_paths,omitempty"`
 }
 
 // Multiple runtime Handlers in a map
@@ -221,6 +227,7 @@ type Runtimes map[string]*RuntimeHandler
 type RuntimeConfig struct {
 	// SeccompUseDefaultWhenEmpty specifies whether the default profile
 	// should be used when an empty one is specified.
+	// This option is currently deprecated, and will be replaced by the SeccompDefault FeatureGate in Kubernetes.
 	SeccompUseDefaultWhenEmpty bool `toml:"seccomp_use_default_when_empty"`
 
 	// NoPivot instructs the runtime to not use `pivot_root`, but instead use `MS_MOVE`
@@ -411,6 +418,10 @@ type RuntimeConfig struct {
 	// EnablePodEvents specifies if the container pod-level events should be generated to optimize the PLEG at Kubelet.
 	EnablePodEvents bool `toml:"enable_pod_events"`
 
+	// IrqBalanceConfigRestoreFile is the irqbalance service banned CPU list to restore.
+	// If empty, no restoration attempt will be done.
+	IrqBalanceConfigRestoreFile string `toml:"irqbalance_config_restore_file"`
+
 	// seccompConfig is the internal seccomp configuration
 	seccompConfig *seccomp.Config
 
@@ -437,6 +448,11 @@ type RuntimeConfig struct {
 
 	// namespaceManager is the internal NamespaceManager configuration
 	namespaceManager *nsmgr.NamespaceManager
+
+	// Whether SELinux should be disabled within a pod,
+	// when it is running in the host network namespace
+	// https://github.com/cri-o/cri-o/issues/5501
+	HostNetworkDisableSELinux bool `toml:"hostnetwork_disable_selinux"`
 }
 
 // ImageConfig represents the "crio.image" TOML config table.
@@ -464,6 +480,14 @@ type ImageConfig struct {
 	// that this be left unspecified so that the default system-wide policy
 	// will be used.
 	SignaturePolicyPath string `toml:"signature_policy"`
+	// SignaturePolicyDir is the root path for pod namespace-separated
+	// signature policies. The final policy to be used on image pull will be
+	// <SIGNATURE_POLICY_DIR>/<NAMESPACE>.json.
+	// If no pod namespace is being provided on image pull (via the sandbox
+	// config), or the concatenated path is non existent, then the
+	// SignaturePolicyPath or system wide policy will be used as fallback.
+	// Must be an absolute path.
+	SignaturePolicyDir string `toml:"signature_policy_dir"`
 	// InsecureRegistries is a list of registries that must be contacted w/o
 	// TLS verification.
 	InsecureRegistries []string `toml:"insecure_registries"`
@@ -793,40 +817,43 @@ func DefaultConfig() (*Config, error) {
 			Runtimes: Runtimes{
 				defaultRuntime: defaultRuntimeHandler(),
 			},
-			SELinux:                    selinuxEnabled(),
-			ApparmorProfile:            apparmor.DefaultProfile,
-			BlockIOConfigFile:          DefaultBlockIOConfigFile,
-			IrqBalanceConfigFile:       DefaultIrqBalanceConfigFile,
-			RdtConfigFile:              rdt.DefaultRdtConfigFile,
-			CgroupManagerName:          cgroupManager.Name(),
-			PidsLimit:                  DefaultPidsLimit,
-			ContainerExitsDir:          containerExitsDir,
-			ContainerAttachSocketDir:   conmonconfig.ContainerAttachSocketDir,
-			MinimumMappableUID:         -1,
-			MinimumMappableGID:         -1,
-			LogSizeMax:                 DefaultLogSizeMax,
-			CtrStopTimeout:             defaultCtrStopTimeout,
-			DefaultCapabilities:        capabilities.Default(),
-			LogLevel:                   "info",
-			HooksDir:                   []string{hooks.DefaultDir},
-			CDISpecDirs:                cdi.DefaultSpecDirs,
-			NamespacesDir:              defaultNamespacesDir,
-			DropInfraCtr:               true,
-			SeccompUseDefaultWhenEmpty: seccompConfig.UseDefaultWhenEmpty(),
-			seccompConfig:              seccomp.New(),
-			apparmorConfig:             apparmor.New(),
-			blockioConfig:              blockio.New(),
-			cgroupManager:              cgroupManager,
-			deviceConfig:               device.New(),
-			namespaceManager:           nsmgr.New(defaultNamespacesDir, ""),
-			rdtConfig:                  rdt.New(),
-			ulimitsConfig:              ulimits.New(),
+			SELinux:                     selinuxEnabled(),
+			ApparmorProfile:             apparmor.DefaultProfile,
+			BlockIOConfigFile:           DefaultBlockIOConfigFile,
+			IrqBalanceConfigFile:        DefaultIrqBalanceConfigFile,
+			RdtConfigFile:               rdt.DefaultRdtConfigFile,
+			CgroupManagerName:           cgroupManager.Name(),
+			PidsLimit:                   DefaultPidsLimit,
+			ContainerExitsDir:           containerExitsDir,
+			ContainerAttachSocketDir:    conmonconfig.ContainerAttachSocketDir,
+			MinimumMappableUID:          -1,
+			MinimumMappableGID:          -1,
+			LogSizeMax:                  DefaultLogSizeMax,
+			CtrStopTimeout:              defaultCtrStopTimeout,
+			DefaultCapabilities:         capabilities.Default(),
+			LogLevel:                    "info",
+			HooksDir:                    []string{hooks.DefaultDir},
+			CDISpecDirs:                 cdi.DefaultSpecDirs,
+			NamespacesDir:               defaultNamespacesDir,
+			DropInfraCtr:                true,
+			SeccompUseDefaultWhenEmpty:  seccompConfig.UseDefaultWhenEmpty(),
+			IrqBalanceConfigRestoreFile: DefaultIrqBalanceConfigRestoreFile,
+			seccompConfig:               seccomp.New(),
+			apparmorConfig:              apparmor.New(),
+			blockioConfig:               blockio.New(),
+			cgroupManager:               cgroupManager,
+			deviceConfig:                device.New(),
+			namespaceManager:            nsmgr.New(defaultNamespacesDir, ""),
+			rdtConfig:                   rdt.New(),
+			ulimitsConfig:               ulimits.New(),
+			HostNetworkDisableSELinux:   true,
 		},
 		ImageConfig: ImageConfig{
-			DefaultTransport: "docker://",
-			PauseImage:       DefaultPauseImage,
-			PauseCommand:     "/pause",
-			ImageVolumes:     ImageVolumesMkdir,
+			DefaultTransport:   "docker://",
+			PauseImage:         DefaultPauseImage,
+			PauseCommand:       "/pause",
+			ImageVolumes:       ImageVolumesMkdir,
+			SignaturePolicyDir: "/etc/crio/policies",
 		},
 		NetworkConfig: NetworkConfig{
 			NetworkDir: cniConfigDir,
@@ -876,6 +903,10 @@ func (c *Config) Validate(onExecution bool) error {
 		filepath.Join(filepath.Dir(c.Listen), "seccomp"),
 	)
 
+	if err := c.ImageConfig.Validate(onExecution); err != nil {
+		return fmt.Errorf("validating image config: %w", err)
+	}
+
 	if err := c.NetworkConfig.Validate(onExecution); err != nil {
 		return fmt.Errorf("validating network config: %w", err)
 	}
@@ -1319,6 +1350,20 @@ func validateExecutablePath(executable, currentPath string) (string, error) {
 	return currentPath, nil
 }
 
+// Validate is the main entry point for image configuration validation.
+// It returns an error on validation failure, otherwise nil.
+func (c *ImageConfig) Validate(onExecution bool) error {
+	if !filepath.IsAbs(c.SignaturePolicyDir) {
+		return fmt.Errorf("signature policy dir %q is not absolute", c.SignaturePolicyDir)
+	}
+	if onExecution {
+		if err := os.MkdirAll(c.SignaturePolicyDir, 0o755); err != nil {
+			return fmt.Errorf("cannot create signature policy dir: %w", err)
+		}
+	}
+	return nil
+}
+
 // Validate is the main entry point for network configuration validation.
 // The parameter `onExecution` specifies if the validation should include
 // execution checks. It returns an `error` on validation failure, otherwise
