@@ -54,13 +54,6 @@ func run() error {
 		return fmt.Errorf("create output path: %w", err)
 	}
 
-	// Get latest release version
-	startTag := util.AddTagPrefix(decVersion(version.Version))
-	logrus.Infof("Using start tag %s", startTag)
-
-	endTag := util.AddTagPrefix(version.Version)
-	logrus.Infof("Using end tag %s", endTag)
-
 	// Generate the notes
 	repo, err := git.OpenRepo(".")
 	if err != nil {
@@ -93,6 +86,8 @@ func run() error {
 	bundleVersion := head
 	shortHead := head[:7]
 	endRev := head
+
+	startTag := util.AddTagPrefix(version.Version)
 	if output, err := command.New(
 		"git", "describe", "--tags", "--exact-match",
 	).RunSilentSuccessOutput(); err == nil {
@@ -101,11 +96,15 @@ func run() error {
 		bundleVersion = foundTag
 		shortHead = foundTag
 		endRev = foundTag
+		startTag = util.AddTagPrefix(decVersion(foundTag))
 	} else {
 		logrus.Infof("Not using git tag because `git describe` failed: %v", err)
 	}
 
-	if _, err := templateFile.WriteString(fmt.Sprintf(`# CRI-O %s
+	logrus.Infof("Using start tag %s", startTag)
+	logrus.Infof("Using end tag %s", endRev)
+
+	if _, err := fmt.Fprintf(templateFile, `# CRI-O %s
 
 The release notes have been generated for the commit range
 [%s...%s](https://github.com/cri-o/cri-o/compare/%s...%s) on %s.
@@ -134,8 +133,10 @@ To verify the artifact signatures via [cosign](https://github.com/sigstore/cosig
 `+"```"+`console
 > export COSIGN_EXPERIMENTAL=1
 > cosign verify-blob cri-o.amd64.%s.tar.gz \
-    --certificate-identity-regexp '.*' \
-    --certificate-oidc-issuer-regexp '.*' \
+    --certificate-identity https://github.com/cri-o/cri-o/.github/workflows/test.yml@refs/tags/%s \
+    --certificate-oidc-issuer https://token.actions.githubusercontent.com \
+    --certificate-github-workflow-repository cri-o/cri-o \
+    --certificate-github-workflow-ref refs/tags/%s \
     --signature cri-o.amd64.%s.tar.gz.sig \
     --certificate cri-o.amd64.%s.tar.gz.cert
 `+"```"+`
@@ -163,7 +164,7 @@ To verify the bill of materials (SBOM) in [SPDX](https://spdx.org) format using
 {{- end -}}
 {{- end -}}
 `,
-		endTag,
+		endRev,
 		startTag, shortHead,
 		startTag, endRev,
 		time.Now().Format(time.RFC1123),
@@ -181,15 +182,16 @@ To verify the bill of materials (SBOM) in [SPDX](https://spdx.org) format using
 		bundleVersion, bundleVersion,
 		bundleVersion, bundleVersion,
 		bundleVersion, bundleVersion,
+		bundleVersion, bundleVersion,
 		bundleVersion, bundleVersion, bundleVersion,
 		bundleVersion, bundleVersion,
 		startTag,
-	)); err != nil {
+	); err != nil {
 		return fmt.Errorf("writing tmplate to file: %w", err)
 	}
 
 	logrus.Infof("Generating release notes")
-	outputFile := endTag + ".md"
+	outputFile := endRev + ".md"
 	outputFilePath := filepath.Join(outputPath, outputFile)
 	os.RemoveAll(outputFilePath)
 	if err := command.Execute(
@@ -235,7 +237,7 @@ To verify the bill of materials (SBOM) in [SPDX](https://spdx.org) format using
 	if err != nil {
 		return fmt.Errorf("open %s file: %w", readmeFile, err)
 	}
-	link := fmt.Sprintf("- [%s](%s)", endTag, outputFile)
+	link := fmt.Sprintf("- [%s](%s)", endRev, outputFile)
 
 	// Item not in list
 	alreadyExistingIndex := indexOfPrefix(link, readmeSlice)
