@@ -117,6 +117,9 @@ func ensureSharedOrSlave(path string, mountInfos []*mount.Info) error {
 }
 
 func addImageVolumes(ctx context.Context, rootfs string, s *Server, containerInfo *storage.ContainerInfo, mountLabel string, specgen *generate.Generator) ([]rspec.Mount, error) {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
+
 	mounts := []rspec.Mount{}
 	for dest := range containerInfo.Config.Config.Volumes {
 		fp, err := securejoin.SecureJoin(rootfs, dest)
@@ -184,6 +187,9 @@ func resolveSymbolicLink(scope, path string) (string, error) {
 
 // setupContainerUser sets the UID, GID and supplemental groups in OCI runtime config
 func setupContainerUser(ctx context.Context, specgen *generate.Generator, rootfs, mountLabel, ctrRunDir string, sc *types.LinuxContainerSecurityContext, imageConfig *v1.Image) error {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
+
 	if sc == nil {
 		return nil
 	}
@@ -294,12 +300,17 @@ func (s *Server) CreateContainer(ctx context.Context, req *types.CreateContainer
 	log.Infof(ctx, "Creating container: %s", translateLabelsToDescription(req.GetConfig().GetLabels()))
 
 	// Check if image is a file. If it is a file it might be a checkpoint archive.
-	checkpointImage := func() bool {
+	checkpointImage, err := func() (bool, error) {
+		if !s.config.CheckpointRestore() {
+			// If CRIU support is not enabled return from
+			// this check as early as possible.
+			return false, nil
+		}
 		if req.Config == nil ||
 			req.Config.Image == nil ||
 			req.SandboxConfig == nil ||
 			req.SandboxConfig.Metadata == nil {
-			return false
+			return false, nil
 		}
 		if _, err := os.Stat(req.Config.Image.Image); err == nil {
 			log.Debugf(
@@ -307,20 +318,28 @@ func (s *Server) CreateContainer(ctx context.Context, req *types.CreateContainer
 				"%q is a file. Assuming it is a checkpoint archive",
 				req.Config.Image.Image,
 			)
-			return true
+			return true, nil
 		}
-		return false
+		// Check if this is an OCI checkpoint image
+		ok, err := s.checkIfCheckpointOCIImage(ctx, req.Config.Image.Image)
+		if err != nil {
+			return false, fmt.Errorf("failed to check if this is a checkpoint image: %w", err)
+		}
+
+		return ok, nil
 	}()
+	if err != nil {
+		return nil, err
+	}
+
 	if checkpointImage {
 		// This might be a checkpoint image. Let's pass
 		// it to the checkpoint code.
 		ctrID, err := s.CRImportCheckpoint(
 			ctx,
-			req.Config.Image.Image,
+			req.Config,
 			req.PodSandboxId,
 			req.SandboxConfig.Metadata.Uid,
-			req.Config.Mounts,
-			req.Config.Annotations,
 		)
 		if err != nil {
 			return nil, err
@@ -332,7 +351,7 @@ func (s *Server) CreateContainer(ctx context.Context, req *types.CreateContainer
 		}, nil
 	}
 
-	sb, err := s.getPodSandboxFromRequest(req.PodSandboxId)
+	sb, err := s.getPodSandboxFromRequest(ctx, req.PodSandboxId)
 	if err != nil {
 		if err == sandbox.ErrIDEmpty {
 			return nil, err
@@ -378,7 +397,7 @@ func (s *Server) CreateContainer(ctx context.Context, req *types.CreateContainer
 		}
 		// if we're able to find the container, and it's created, this is actually a duplicate request
 		// Just return that container
-		if reservedCtr := s.GetContainer(reservedID); reservedCtr != nil && reservedCtr.Created() {
+		if reservedCtr := s.GetContainer(ctx, reservedID); reservedCtr != nil && reservedCtr.Created() {
 			return &types.CreateContainerResponse{ContainerId: reservedID}, nil
 		}
 		cachedID, resourceErr := s.getResourceOrWait(ctx, ctr.Name(), "container")
@@ -388,10 +407,10 @@ func (s *Server) CreateContainer(ctx context.Context, req *types.CreateContainer
 		return nil, fmt.Errorf("%v: %w", resourceErr, err)
 	}
 
-	s.resourceStore.SetStageForResource(ctr.Name(), "container creating")
+	s.resourceStore.SetStageForResource(ctx, ctr.Name(), "container creating")
 
 	resourceCleaner.Add(ctx, "createCtr: releasing container name "+ctr.Name(), func() error {
-		s.ReleaseContainerName(ctr.Name())
+		s.ReleaseContainerName(ctx, ctr.Name())
 		return nil
 	})
 
@@ -400,15 +419,15 @@ func (s *Server) CreateContainer(ctx context.Context, req *types.CreateContainer
 		return nil, err
 	}
 	resourceCleaner.Add(ctx, "createCtr: deleting container "+ctr.ID()+" from storage", func() error {
-		if err := s.StorageRuntimeServer().DeleteContainer(ctr.ID()); err != nil {
+		if err := s.StorageRuntimeServer().DeleteContainer(ctx, ctr.ID()); err != nil {
 			return fmt.Errorf("failed to cleanup container storage: %w", err)
 		}
 		return nil
 	})
 
-	s.addContainer(newContainer)
+	s.addContainer(ctx, newContainer)
 	resourceCleaner.Add(ctx, "createCtr: removing container "+newContainer.ID(), func() error {
-		s.removeContainer(newContainer)
+		s.removeContainer(ctx, newContainer)
 		return nil
 	})
 
@@ -422,12 +441,12 @@ func (s *Server) CreateContainer(ctx context.Context, req *types.CreateContainer
 		return nil
 	})
 
-	mappings, err := s.getSandboxIDMappings(sb)
+	mappings, err := s.getSandboxIDMappings(ctx, sb)
 	if err != nil {
 		return nil, err
 	}
 
-	s.resourceStore.SetStageForResource(ctr.Name(), "container runtime creation")
+	s.resourceStore.SetStageForResource(ctx, ctr.Name(), "container runtime creation")
 	if err := s.createContainerPlatform(ctx, newContainer, sb.CgroupParent(), mappings); err != nil {
 		return nil, err
 	}
@@ -455,6 +474,12 @@ func (s *Server) CreateContainer(ctx context.Context, req *types.CreateContainer
 
 	newContainer.SetCreated()
 
+	if err := s.nri.postCreateContainer(ctx, sb, newContainer); err != nil {
+		log.Warnf(ctx, "NRI post-create event failed for container %q: %v",
+			newContainer.ID(), err)
+	}
+	s.generateCRIEvent(ctx, newContainer, types.ContainerEventType_CONTAINER_CREATED_EVENT)
+
 	log.Infof(ctx, "Created container %s: %s", newContainer.ID(), newContainer.Description())
 	return &types.CreateContainerResponse{
 		ContainerId: ctr.ID(),
