@@ -14,6 +14,7 @@ import (
 	"github.com/golang/mock/gomock"
 	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
+	v1 "github.com/opencontainers/image-spec/specs-go/v1"
 	specs "github.com/opencontainers/runtime-spec/specs-go"
 	types "k8s.io/cri-api/pkg/apis/runtime/v1"
 )
@@ -93,7 +94,7 @@ var _ = t.Describe("ContainerRestore", func() {
 				"",
 			)
 			// Then
-			Expect(err.Error()).To(ContainSubstring(`failed to read "spec.dump": failed to read`))
+			Expect(err.Error()).To(ContainSubstring(`failed to read "spec.dump": `))
 		})
 	})
 	t.Describe("ContainerRestore from archive into new pod", func() {
@@ -322,6 +323,203 @@ var _ = t.Describe("ContainerRestore", func() {
 			Expect(err.Error()).To(Equal(`failed to read "io.kubernetes.cri-o.Annotations": unexpected end of JSON input`))
 		})
 	})
+	t.Describe("ContainerRestore from archive into new pod", func() {
+		It("should fail because archive contains no io.kubernetes.cri-o.Labels", func() {
+			// Given
+			addContainerAndSandbox()
+			testContainer.SetStateAndSpoofPid(&oci.ContainerState{
+				State: specs.State{Status: oci.ContainerStateRunning},
+			})
+
+			err := os.WriteFile(
+				"spec.dump",
+				[]byte(
+					`{"annotations":{"io.kubernetes.cri-o.Metadata"`+
+						`:"{\"name\":\"container-to-restore\"}",`+
+						`"io.kubernetes.cri-o.Annotations": "{\"name\":\"NAME\"}"}}`),
+				0o644,
+			)
+			Expect(err).To(BeNil())
+			defer os.RemoveAll("spec.dump")
+			err = os.WriteFile("config.dump", []byte(`{"rootfsImageName": "image"}`), 0o644)
+			Expect(err).To(BeNil())
+			defer os.RemoveAll("config.dump")
+			outFile, err := os.Create("archive.tar")
+			Expect(err).To(BeNil())
+			defer outFile.Close()
+			input, err := archive.TarWithOptions(".", &archive.TarOptions{
+				Compression:      archive.Uncompressed,
+				IncludeSourceDir: true,
+				IncludeFiles:     []string{"spec.dump", "config.dump"},
+			})
+			Expect(err).To(BeNil())
+			defer os.RemoveAll("archive.tar")
+			_, err = io.Copy(outFile, input)
+			Expect(err).To(BeNil())
+			containerConfig := &types.ContainerConfig{
+				Image: &types.ImageSpec{
+					Image: "archive.tar",
+				},
+			}
+			// When
+
+			_, err = sut.CRImportCheckpoint(
+				context.Background(),
+				containerConfig,
+				"",
+				"",
+			)
+
+			// Then
+			Expect(err.Error()).To(Equal(`failed to read "io.kubernetes.cri-o.Labels": unexpected end of JSON input`))
+		})
+	})
+	t.Describe("ContainerRestore from archive into new pod", func() {
+		It("should fail with 'PodSandboxId should not be empty'", func() {
+			// Given
+			addContainerAndSandbox()
+			testContainer.SetStateAndSpoofPid(&oci.ContainerState{
+				State: specs.State{Status: oci.ContainerStateRunning},
+			})
+
+			err := os.WriteFile(
+				"spec.dump",
+				[]byte(
+					`{"annotations":{"io.kubernetes.cri-o.Metadata"`+
+						`:"{\"name\":\"container-to-restore\"}",`+
+						`"io.kubernetes.cri-o.Annotations": "{\"name\":\"NAME\"}",`+
+						`"io.kubernetes.cri-o.Labels": "{\"io.kubernetes.container.name\":\"counter\"}"}}`),
+				0o644,
+			)
+			Expect(err).To(BeNil())
+			defer os.RemoveAll("spec.dump")
+			err = os.WriteFile("config.dump", []byte(`{"rootfsImageName": "image"}`), 0o644)
+			Expect(err).To(BeNil())
+			defer os.RemoveAll("config.dump")
+			outFile, err := os.Create("archive.tar")
+			Expect(err).To(BeNil())
+			defer outFile.Close()
+			input, err := archive.TarWithOptions(".", &archive.TarOptions{
+				Compression:      archive.Uncompressed,
+				IncludeSourceDir: true,
+				IncludeFiles:     []string{"spec.dump", "config.dump"},
+			})
+			Expect(err).To(BeNil())
+			defer os.RemoveAll("archive.tar")
+			_, err = io.Copy(outFile, input)
+			Expect(err).To(BeNil())
+			containerConfig := &types.ContainerConfig{
+				Image: &types.ImageSpec{
+					Image: "archive.tar",
+				},
+			}
+			// When
+
+			_, err = sut.CRImportCheckpoint(
+				context.Background(),
+				containerConfig,
+				"",
+				"",
+			)
+
+			// Then
+			Expect(err.Error()).To(Equal(`PodSandboxId should not be empty`))
+		})
+	})
+	t.Describe("ContainerRestore from archive into new pod", func() {
+		It("should succeed", func() {
+			// Given
+			addContainerAndSandbox()
+			testContainer.SetStateAndSpoofPid(&oci.ContainerState{
+				State: specs.State{Status: oci.ContainerStateRunning},
+			})
+
+			err := os.WriteFile(
+				"spec.dump",
+				[]byte(`{"annotations":{"io.kubernetes.cri-o.Metadata"`+
+					`:"{\"name\":\"container-to-restore\"}",`+
+					`"io.kubernetes.cri-o.Annotations": "{\"name\":\"NAME\"}",`+
+					`"io.kubernetes.cri-o.Labels": "{\"io.kubernetes.container.name\":\"counter\"}",`+
+					`"io.kubernetes.cri-o.SandboxID": "sandboxID"},`+
+					`"mounts": [{"destination": "/proc"},`+
+					`{"destination":"/data","source":"/data","options":`+
+					`["rw","ro","rbind","rprivate","rshared","rslaved"]}],`+
+					`"linux": {"maskedPaths": ["/proc/acpi"], "readonlyPaths": ["/proc/asound"]}}`),
+				0o644,
+			)
+			Expect(err).To(BeNil())
+			defer os.RemoveAll("spec.dump")
+			err = os.WriteFile("config.dump", []byte(`{"rootfsImageName": "image"}`), 0o644)
+			Expect(err).To(BeNil())
+			defer os.RemoveAll("config.dump")
+			outFile, err := os.Create("archive.tar")
+			Expect(err).To(BeNil())
+			defer outFile.Close()
+			input, err := archive.TarWithOptions(".", &archive.TarOptions{
+				Compression:      archive.Uncompressed,
+				IncludeSourceDir: true,
+				IncludeFiles:     []string{"spec.dump", "config.dump"},
+			})
+			Expect(err).To(BeNil())
+			defer os.RemoveAll("archive.tar")
+			_, err = io.Copy(outFile, input)
+			Expect(err).To(BeNil())
+			containerConfig := &types.ContainerConfig{
+				Image: &types.ImageSpec{
+					Image: "archive.tar",
+				},
+				Linux: &types.LinuxContainerConfig{
+					Resources:       &types.LinuxContainerResources{},
+					SecurityContext: &types.LinuxContainerSecurityContext{},
+				},
+			}
+
+			size := uint64(100)
+			gomock.InOrder(
+				imageServerMock.EXPECT().ResolveNames(
+					gomock.Any(), gomock.Any()).
+					Return([]string{"image"}, nil),
+
+				imageServerMock.EXPECT().ImageStatus(
+					gomock.Any(), gomock.Any()).
+					Return(&storage.ImageResult{
+						ID:   "image",
+						User: "10", Size: &size,
+						Annotations: map[string]string{
+							crioann.CheckpointAnnotationName: "foo",
+						},
+					}, nil),
+
+				runtimeServerMock.EXPECT().CreateContainer(gomock.Any(), gomock.Any(),
+					gomock.Any(), gomock.Any(), gomock.Any(), gomock.Any(),
+					gomock.Any(), gomock.Any(), gomock.Any(), gomock.Any(),
+					gomock.Any(), gomock.Any()).
+					Return(storage.ContainerInfo{
+						Config: &v1.Image{
+							Config: v1.ImageConfig{
+								Entrypoint: []string{"sh"},
+							},
+						},
+					},
+						nil,
+					),
+				runtimeServerMock.EXPECT().StartContainer(gomock.Any()).
+					Return(emptyDir, nil),
+			)
+
+			// When
+
+			_, err = sut.CRImportCheckpoint(
+				context.Background(),
+				containerConfig,
+				"",
+				"",
+			)
+
+			// Then
+			Expect(err).To(BeNil())
+		})
+	})
 	t.Describe("ContainerRestore from OCI archive", func() {
 		It("should fail because archive does not exist", func() {
 			// Given
@@ -373,7 +571,7 @@ var _ = t.Describe("ContainerRestore", func() {
 			)
 
 			// Then
-			Expect(err.Error()).To(ContainSubstring(`failed to read spec.dump: open spec.dump: no such file or directory`))
+			Expect(err.Error()).To(ContainSubstring(`failed to read "spec.dump": open spec.dump: no such file or directory`))
 		})
 	})
 })
