@@ -30,15 +30,15 @@ import (
 	"path"
 	"time"
 
+	"github.com/containers/common/pkg/resize"
+	remotecommandserver "github.com/cri-o/cri-o/server/streaming/remotecommand"
 	restful "github.com/emicklei/go-restful"
 	"google.golang.org/grpc/codes"
 	"google.golang.org/grpc/status"
 	apiTypes "k8s.io/apimachinery/pkg/types"
 	remotecommandconsts "k8s.io/apimachinery/pkg/util/remotecommand"
-	"k8s.io/client-go/tools/remotecommand"
 	types "k8s.io/cri-api/pkg/apis/runtime/v1"
 	"k8s.io/kubernetes/pkg/kubelet/cri/streaming/portforward"
-	remotecommandserver "k8s.io/kubernetes/pkg/kubelet/cri/streaming/remotecommand"
 )
 
 // Server is the library interface to serve the stream requests.
@@ -63,8 +63,8 @@ type Server interface {
 
 // Runtime is the interface to execute the commands and provide the streams.
 type Runtime interface {
-	Exec(ctx context.Context, containerID string, cmd []string, in io.Reader, out, err io.WriteCloser, tty bool, resize <-chan remotecommand.TerminalSize) error
-	Attach(ctx context.Context, containerID string, in io.Reader, out, err io.WriteCloser, tty bool, resize <-chan remotecommand.TerminalSize) error
+	Exec(ctx context.Context, containerID string, cmd []string, in io.Reader, out, err io.WriteCloser, tty bool, resizeChan <-chan resize.TerminalSize) error
+	Attach(ctx context.Context, containerID string, in io.Reader, out, err io.WriteCloser, tty bool, resizeChan <-chan resize.TerminalSize) error
 	PortForward(ctx context.Context, podSandboxID string, port int32, stream io.ReadWriteCloser) error
 }
 
@@ -373,12 +373,12 @@ var (
 	_ portforward.PortForwarder    = &criAdapter{}
 )
 
-func (a *criAdapter) ExecInContainer(ctx context.Context, podName string, podUID apiTypes.UID, container string, cmd []string, in io.Reader, out, err io.WriteCloser, tty bool, resize <-chan remotecommand.TerminalSize, timeout time.Duration) error {
-	return a.Runtime.Exec(ctx, container, cmd, in, out, err, tty, resize)
+func (a *criAdapter) ExecInContainer(ctx context.Context, podName string, podUID apiTypes.UID, container string, cmd []string, in io.Reader, out, err io.WriteCloser, tty bool, resizeChan <-chan resize.TerminalSize, timeout time.Duration) error {
+	return a.Runtime.Exec(ctx, container, cmd, in, out, err, tty, resizeChan)
 }
 
-func (a *criAdapter) AttachContainer(ctx context.Context, podName string, podUID apiTypes.UID, container string, in io.Reader, out, err io.WriteCloser, tty bool, resize <-chan remotecommand.TerminalSize) error {
-	return a.Runtime.Attach(ctx, container, in, out, err, tty, resize)
+func (a *criAdapter) AttachContainer(ctx context.Context, podName string, podUID apiTypes.UID, container string, in io.Reader, out, err io.WriteCloser, tty bool, resizeChan <-chan resize.TerminalSize) error {
+	return a.Runtime.Attach(ctx, container, in, out, err, tty, resizeChan)
 }
 
 func (a *criAdapter) PortForward(ctx context.Context, podName string, podUID apiTypes.UID, port int32, stream io.ReadWriteCloser) error {
